

set -ex



python -c "import numpy; numpy._pytesttester._show_numpy_info()"
python -c "import scipy, sys; sys.exit(not scipy.test(verbose=2, label='full', tests=None, extra_argv=['-k', 'not (_not_a_real_test)', '--timeout=1200', '--durations=50']))"
exit 0
