print("import: 'scipy'")
import scipy

print("import: 'scipy.cluster._hierarchy'")
import scipy.cluster._hierarchy

print("import: 'scipy.cluster._vq'")
import scipy.cluster._vq

print("import: 'scipy.fft'")
import scipy.fft

print("import: 'scipy.integrate._dop'")
import scipy.integrate._dop

print("import: 'scipy.integrate._odepack'")
import scipy.integrate._odepack

print("import: 'scipy.integrate._quadpack'")
import scipy.integrate._quadpack

print("import: 'scipy.integrate._test_multivariate'")
import scipy.integrate._test_multivariate

print("import: 'scipy.integrate._test_odeint_banded'")
import scipy.integrate._test_odeint_banded

print("import: 'scipy.integrate.lsoda'")
import scipy.integrate.lsoda

print("import: 'scipy.integrate.vode'")
import scipy.integrate.vode

print("import: 'scipy.interpolate._fitpack'")
import scipy.interpolate._fitpack

print("import: 'scipy.interpolate._ppoly'")
import scipy.interpolate._ppoly

print("import: 'scipy.interpolate.dfitpack'")
import scipy.interpolate.dfitpack

print("import: 'scipy.interpolate.interpnd'")
import scipy.interpolate.interpnd

print("import: 'scipy.io.matlab.mio5_utils'")
import scipy.io.matlab.mio5_utils

print("import: 'scipy.io.matlab.mio_utils'")
import scipy.io.matlab.mio_utils

print("import: 'scipy.io.matlab.streams'")
import scipy.io.matlab.streams

print("import: 'scipy.linalg._decomp_update'")
import scipy.linalg._decomp_update

print("import: 'scipy.linalg._fblas'")
import scipy.linalg._fblas

print("import: 'scipy.linalg._flapack'")
import scipy.linalg._flapack

print("import: 'scipy.linalg._flinalg'")
import scipy.linalg._flinalg

print("import: 'scipy.linalg._interpolative'")
import scipy.linalg._interpolative

print("import: 'scipy.linalg._solve_toeplitz'")
import scipy.linalg._solve_toeplitz

print("import: 'scipy.linalg.cython_blas'")
import scipy.linalg.cython_blas

print("import: 'scipy.linalg.cython_lapack'")
import scipy.linalg.cython_lapack

print("import: 'scipy.ndimage._nd_image'")
import scipy.ndimage._nd_image

print("import: 'scipy.ndimage._ni_label'")
import scipy.ndimage._ni_label

print("import: 'scipy.odr.__odrpack'")
import scipy.odr.__odrpack

print("import: 'scipy.optimize._cobyla'")
import scipy.optimize._cobyla

print("import: 'scipy.optimize._group_columns'")
import scipy.optimize._group_columns

print("import: 'scipy.optimize._lbfgsb'")
import scipy.optimize._lbfgsb

print("import: 'scipy.optimize._lsq.givens_elimination'")
import scipy.optimize._lsq.givens_elimination

print("import: 'scipy.optimize._minpack'")
import scipy.optimize._minpack

print("import: 'scipy.optimize._nnls'")
import scipy.optimize._nnls

print("import: 'scipy.optimize._slsqp'")
import scipy.optimize._slsqp

print("import: 'scipy.optimize._zeros'")
import scipy.optimize._zeros

print("import: 'scipy.optimize.minpack2'")
import scipy.optimize.minpack2

print("import: 'scipy.optimize.moduleTNC'")
import scipy.optimize.moduleTNC

print("import: 'scipy.signal._max_len_seq_inner'")
import scipy.signal._max_len_seq_inner

print("import: 'scipy.signal._spectral'")
import scipy.signal._spectral

print("import: 'scipy.signal.sigtools'")
import scipy.signal.sigtools

print("import: 'scipy.signal.spline'")
import scipy.signal.spline

print("import: 'scipy.sparse._csparsetools'")
import scipy.sparse._csparsetools

print("import: 'scipy.sparse._sparsetools'")
import scipy.sparse._sparsetools

print("import: 'scipy.sparse.csgraph._min_spanning_tree'")
import scipy.sparse.csgraph._min_spanning_tree

print("import: 'scipy.sparse.csgraph._reordering'")
import scipy.sparse.csgraph._reordering

print("import: 'scipy.sparse.csgraph._shortest_path'")
import scipy.sparse.csgraph._shortest_path

print("import: 'scipy.sparse.csgraph._tools'")
import scipy.sparse.csgraph._tools

print("import: 'scipy.sparse.csgraph._traversal'")
import scipy.sparse.csgraph._traversal

print("import: 'scipy.sparse.linalg.dsolve._superlu'")
import scipy.sparse.linalg.dsolve._superlu

print("import: 'scipy.sparse.linalg.eigen.arpack._arpack'")
import scipy.sparse.linalg.eigen.arpack._arpack

print("import: 'scipy.sparse.linalg.isolve._iterative'")
import scipy.sparse.linalg.isolve._iterative

print("import: 'scipy.spatial._distance_wrap'")
import scipy.spatial._distance_wrap

print("import: 'scipy.spatial.ckdtree'")
import scipy.spatial.ckdtree

print("import: 'scipy.spatial.qhull'")
import scipy.spatial.qhull

print("import: 'scipy.special'")
import scipy.special

print("import: 'scipy.special._ellip_harm_2'")
import scipy.special._ellip_harm_2

print("import: 'scipy.special._ufuncs'")
import scipy.special._ufuncs

print("import: 'scipy.special._ufuncs_cxx'")
import scipy.special._ufuncs_cxx

print("import: 'scipy.special.specfun'")
import scipy.special.specfun

print("import: 'scipy.stats'")
import scipy.stats

print("import: 'scipy.stats.mvn'")
import scipy.stats.mvn

print("import: 'scipy.stats.statlib'")
import scipy.stats.statlib

